<?php
/*-------------------------------------------------------
*
*   LiveStreet Engine Social Networking
*   Copyright © 2008 Mzhelskiy Maxim
*
*--------------------------------------------------------
*
*   Official site: www.livestreet.ru
*   Contact e-mail: rus.engine@gmail.com
*
*   GNU General Public License, version 2:
*   http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
*
---------------------------------------------------------
*/

/**
 * Плагин для смарти.
 * Позволяет получать дату с возможностью склонения
 * формы слова и поддержкой мультиязычноти.
 *
 * Список ключей параметров:
 * 		date*          [string]
 * 		format*        [string]
 * 		declination*   [int]
 * 		now*           [int]    Количество секунд, в течении которых событие имеет статус "Только что"
 * 		day*   		   [string] Указывает на необходимость замены "Сегодня", "Вчера", "Завтра".
 * 								В указанном формате 'day' будет заменено на соответствующее значение.
 * 		minutes_back*  [int]    Количество минут, в течении которых событие имеет статус "... минут назад"
 * 		hours_back*    [int]    Количество часов, в течении которых событие имеет статус "... часов назад"
 *      timezone*       [DateTimeZone]  объект часового пояса, с учётом которого показывается дата/время. Если не задан, берётся из настроек текущего пользователя.
 *
 * (* - параметр является необязательным)
 *
 * @param   array $aParams
 * @param   Smarty $oSmarty
 * @return  string
 */
function smarty_function_date_format($aParams, &$oSmarty)
{
    require_once(Config::Get('path.root.engine').'/classes/Engine.class.php');
    $oEngine = Engine::getInstance();
    /** @var ModuleLang $lang */
    $lang = $oEngine->GetModuleObject('Lang');

    $sFormatDefault = "d F Y, H:i";  //  формат даты по умолчанию
    $sFormat = (empty($aParams['format'])) ? $sFormatDefault : $aParams['format'];
    $iDeclinationDefault = 1;       //  индекс склонения по умолчанию
    $iDeclination = (!isset($aParams['declination'])) ? $iDeclinationDefault : $aParams['declination'];
    // Если указан другой язык, подгружаем его
    if(isset($aParams['lang']) and $aParams['lang']!=$lang->GetLang())
        $lang->SetLang($aParams['lang']);

    // Определяем дату
    $oDate = (new DateTime((is_numeric($aParams['date']) ? '@':'').$aParams['date'] ?? 'now'));
    // Применяем переданный часовой пояс или часовой пояс пользователя, если задан (иначе работаем в часовом поясе сервера)
    if(isset($aParams['timezone']) && is_a($aParams['timezone'],DateTimeZone::class)) {
        $oDate->setTimezone($aParams['timezone']);
    } else {
        /** @var ModuleUser_EntityUser $oUserCurrent */
        $oUserCurrent = $oEngine->GetModuleObject('User')->GetUserCurrent();
        if(!empty($oUserCurrent))
            $oDate->setTimezone($oUserCurrent->getSettingsTimezoneObject());
        unset($oUserCurrent);
    }
    // Формируем заданную и текущую даты в виде числа секунд с UNIX Epoch, но с учётом часового пояса для подсчётов относительных дат ниже
    $iDate = $oDate->getTimestamp()+$oDate->getOffset();
    $iNow = time()+$oDate->getOffset();

    // Если указана необходимость выполнять проверку на NOW
    if(isset($aParams['now'])) {
        if($iDate+$aParams['now']>$iNow) {
            return $lang->Get('date_now');
        }
    }

    // Если указана необходимость на проверку minutes back
    if(isset($aParams['minutes_back'])) {
        $iTimeDelta = round(($iNow - $iDate)/60);
        if($iTimeDelta<$aParams['minutes_back']) {
            require_once(__DIR__.'/modifier.declension.php');
            return ($iTimeDelta!=0)
                ? smarty_modifier_declension(
                    $iTimeDelta,
                    $lang->Get('date_minutes_back', ['minutes'=>$iTimeDelta]),
                    $lang->GetLang()
                )
                : $lang->Get('date_minutes_back_less');
        }
    }

    // Если указана необходимость на проверку hours back
    if(isset($aParams['hours_back'])) {
        $iTimeDelta = round(($iNow - $iDate)/(60*60));
        if($iTimeDelta<$aParams['hours_back']) {
            require_once(__DIR__.'/modifier.declension.php');
            return ($iTimeDelta!=0)
                ? smarty_modifier_declension(
                    $iTimeDelta,
                    $lang->Get('date_hours_back', ['hours'=>$iTimeDelta]),
                    $lang->GetLang()
                )
                : $lang->Get('date_hours_back_less');
        }
    }

    // Если указана необходимость автоподстановки "Сегодня", "Вчера", "Завтра".
    if(!empty($aParams['day'])) {
        $iDateOnly = floor($iDate / (24*60*60));
        $iNowOnly = floor($iNow / (24*60*60));
        if(abs($iNowOnly - $iDateOnly) <= 1) {
            switch($iDateOnly) {
                // Если дата совпадает с сегодняшней
                case $iNowOnly:
                    $sDay=$lang->Get('date_today');
                    break;
                // Если дата совпадает со вчерашней
                case $iNowOnly-1:
                    $sDay=$lang->Get('date_yesterday');
                    break;
                // Если дата совпадает с завтрашней
                case $iNowOnly+1:
                    $sDay=$lang->Get('date_tomorrow');
                    break;
                default:
                    $sDay=null;
            }
            if($sDay) {
                // fixme: для языков, использующих латинские буквы, необходимо экранирование, например, preg_replace("#(\w{1})#", '\\\${1}', $sDay)
                $sFormat=str_replace("day", $sDay, $aParams['day']);
                return $oDate->format($sFormat);
            }
        }
    }

    if(strpos($sFormat,'F') !== false) {
        // Определяем нужное текстовое значение названия месяца
        $iMonth = $oDate->format('n');
        $sMonth = $lang->Get('month_array')[$iMonth] ?? "";
        // Если не найден индекс склонения, берем склонене по умолчанию.
        // Если индекс по умолчанию также не определен, берем первое значение в массиве.
        if(is_array($sMonth))
            $sMonth = $sMonth[$iDeclination] ?? $sMonth[$iDeclinationDefault];
        // Сначала замена "месяца" на "\м\е\с\я\ц\а", потом замена F (но не \F) на это.
        // fixme: для языков, использующих латинские буквы, необходимо экранирование, например, preg_replace("#(\w{1})#", '\\\${1}', sMonth)
        $sFormat=preg_replace('~(^F)|([^\\\])F~', '${2}'.$sMonth, $sFormat);
    }

    return $oDate->format($sFormat);
}
