import React from 'react';

import Cropper from "react-cropper";

import Modal from 'components/Modal';
import FileInput from 'components/FileInput';
import {gettext} from 'core/lang';

import utils from './utils';
import {error} from 'core/messages';


export default class ImageUploader extends React.Component {
    editor = null; input = null;
    state = {
        sourceImg: null,
        croppedImg: this.props.img,
        defaultImg: this.props.defaultImg,
        editorOpened: false,
        uploading:false,
        removing:false,
        dragging:false,
        imgIsEmpty: this.props.imgIsEmpty,
    };
    static propTypes = {
        width: React.PropTypes.number,
        height: React.PropTypes.number,
        border: React.PropTypes.number,
        title: React.PropTypes.string,
        containerClass: React.PropTypes.string,
        previewClass: React.PropTypes.string,
        linkClass: React.PropTypes.string,
        onUpload: React.PropTypes.func,
        onRemove: React.PropTypes.func,
        aspectRatio: React.PropTypes.number,
        imgIsEmpty: React.PropTypes.bool,
        maxWidth: React.PropTypes.number,
        minImageWidth: React.PropTypes.number,
        minImageHeight: React.PropTypes.number,
    };
    sourceImageType = '';
    sourceImageSize = [0,0];
    sourceImageInitialScaleRatio = 1;
    sourceImageCrop = null;
    tempImage = null;
    tempImageType = '';
    tempImageSize = [0,0];
    tempImageInitialScaleRatio = 1;
    tempImageCrop = null;

    setEditorRef = (editor) => {
        if (editor) this.editor = editor;
    };
    setInputRef = (input) => {
        if(input) this.input = input;
    };
    resetCropperMinSize = (cropBoxScale) => {
        cropBoxScale = cropBoxScale || this.editor.getCanvasData().width / this.editor.getCanvasData().naturalWidth;
        const minw = this.props.minImageWidth*cropBoxScale > 20 ? this.props.minImageWidth*cropBoxScale : 20;
        const minh = this.props.minImageHeight*cropBoxScale > 20 ? this.props.minImageHeight*cropBoxScale : 20;
        this.editor.cropper.options.minCropBoxWidth = minw;
        this.editor.cropper.options.minCropBoxHeight = minh;
        this.editor.cropper.cropBoxData.minWidth = minw;
        this.editor.cropper.cropBoxData.minHeight = minh;
    };
    onCropperReady = () => {
        this.resetCropperMinSize();
        this.tempImageInitialScaleRatio = this.editor.getCanvasData().width / this.editor.getCanvasData().naturalWidth;
    };
    onCropperZoom = (event) => {
        if(event.detail.ratio === event.detail.oldRatio || event.detail.ratio > 1 || event.detail.ratio < this.tempImageInitialScaleRatio) {
            event.preventDefault(); return false; }
        const cropBoxScale = event.detail.ratio / event.detail.oldRatio;
        const cropBoxData = this.editor.getCropBoxData();

        cropBoxData['new_width'] = cropBoxData['width']*cropBoxScale;
        cropBoxData['new_height'] = cropBoxData['height']*cropBoxScale;
        if(event.detail.originalEvent) {
            var pointers = this.editor.cropper.pointers;
            var offset = {
                left: this.editor.cropper.cropBox.getBoundingClientRect().left + (window.scrollX - document.documentElement.clientLeft),
                top: this.editor.cropper.cropBox.getBoundingClientRect().top + (window.scrollY - document.documentElement.clientTop)
            };
            function getPointersCenter(pointers) {
                var pageX = 0;
                var pageY = 0;
                var count = 0;
                forEach(pointers, function (_ref3) {
                    var startX = _ref3.startX,
                        startY = _ref3.startY;
                    pageX += startX;
                    pageY += startY;
                    count += 1;
                });
                pageX /= count;
                pageY /= count;
                return {
                    pageX: pageX,
                    pageY: pageY
                };
            }
            var center = pointers && Object.keys(pointers).length ? getPointersCenter(pointers) : {
                pageX: event.detail.originalEvent.pageX,
                pageY: event.detail.originalEvent.pageY
            }; // Zoom from the triggering point of the event
            cropBoxData['left'] -= (cropBoxData['new_width'] - cropBoxData['width']) * ((center.pageX - offset.left) / cropBoxData['width']);
            cropBoxData['top'] -= (cropBoxData['new_height'] - cropBoxData['height']) * ((center.pageY - offset.top) / cropBoxData['height']);
        }
        this.editor.setCropBoxData({top:cropBoxData['top'],left:cropBoxData['left'],
            width:cropBoxData['new_width'],height:cropBoxData['new_height']});
        this.resetCropperMinSize(event.detail.ratio);
    };

    handleFileChange = (blob/*dataURI*/, width, height) => {
        if(this.props.minImageWidth && width < this.props.minImageWidth) {
            error(gettext('common_error'),gettext('image_uploader_error_too_small_width').replace(new RegExp('%%minsize%%', 'g'),this.props.minImageWidth)); return;
        }
        if(this.props.minImageHeight && height < this.props.minImageHeight) {
            error(gettext('common_error'),gettext('image_uploader_error_too_small_height').replace(new RegExp('%%minsize%%', 'g'),this.props.minImageHeight)); return;
        }
        this.tempImage = URL.createObjectURL(blob);//dataURI;
        this.tempImageType = blob.type;
        this.tempImageSize = [width,height];
        this.tempImageCrop = null;
        this.setState({
            //sourceImg: dataURI,
            editorOpened: true
        });
    };

    handleFileDrop = (e) => {
        e.stopPropagation(); e.preventDefault();
        this.setState({dragging:false});
        const files = e.dataTransfer.files;
        if(files === undefined || !files.length) return;
        if(!files[0].type.startsWith('image/')) return;
        this.input.handleUpload({target:{files:[files[0]]}});
    };
    handleDragEnter = (e) => {
        e.stopPropagation(); e.preventDefault();
        const items = e.dataTransfer.items;
        if(items === undefined || !items.length) return;
        if(items[0].kind !== 'file' || !items[0].type.startsWith('image/')) return;
        this.setState({dragging:true});
    };
    handleDragOver = (e) => {
        e.stopPropagation(); e.preventDefault();
    };
    handleDragLeave = (e) => {
        e.stopPropagation(); e.preventDefault();
        this.setState({dragging:false});
    };

    handleRequestOpen = () => {
        if(this.state.sourceImg)
            this.setState({editorOpened: true});
    };

    handleRequestClose = (restoreSource) => {
        if(restoreSource !== false && this.tempImage !== this.state.sourceImg) {
            URL.revokeObjectURL(this.tempImage);
            this.tempImage = this.state.sourceImg;
            this.tempImageType = this.sourceImageType;
            this.tempImageSize = this.sourceImageSize;
            this.tempImageInitialScaleRatio = this.sourceImageInitialScaleRatio;
            this.tempImageCrop = this.sourceImageCrop;
        }
        this.setState({editorOpened: false});
    };

    handleSave = () => {
        if(this.state.uploading) return;
        this.setState({uploading:true});
        /*const canvas = this.editor.getCroppedCanvas({
            maxWidth: 4096,//Limitations.processingImgMaxWidth, // /templates/skin/synio/footer.tpl:46, /settings/application.php:27
            maxHeight: 4096,//Limitations.processingImgMaxHeight, // /templates/skin/synio/footer.tpl:47, /settings/application.php:28
            imageSmoothingEnabled: false,
            imageSmoothingQuality: 'high',
        });*/
        const tempImageCrop = this.editor.getData(true);
        const canvas = document.createElement("canvas");
        const ctx = canvas.getContext("2d");
        canvas.width = Math.min(tempImageCrop['width'],2048);
        canvas.height = Math.min(tempImageCrop['height'],2048);
        ctx.drawImage(this.editor.cropper.image,tempImageCrop['x'],tempImageCrop['y'],tempImageCrop['width'],tempImageCrop['height'],0,0,canvas.width,canvas.height);
        const croppedImage = canvas.toDataURL(this.tempImageType);
        const result = this.props.onUpload(utils.extractBase64(croppedImage));

        if(!(result instanceof Promise))
            throw new Error('ImageUploader handleSave: wanted Promise, got '+result);

        result
            .then((data) => {
                this.setState({sourceImg:this.tempImage, croppedImg:croppedImage, imgIsEmpty:false, uploading:false});
                this.sourceImageType = this.tempImageType;
                this.sourceImageSize = this.tempImageSize;
                this.sourceImageInitialScaleRatio = this.tempImageInitialScaleRatio;
                this.sourceImageCrop = this.tempImageCrop = tempImageCrop;
                this.handleRequestClose(false);
            })
            .catch((data) => { console.error(data); this.setState({uploading:false}); this.handleRequestClose(false); })
    };

    handleRemove = () => {
        if(this.props.onRemove && !this.state.removing) {
            this.setState({removing:true});
            const result = this.props.onRemove();
            if(!(result instanceof Promise))
                throw new Error('ImageUploader handleRemove: wanted Promise, got '+result);
            result
                .then((data) => {
                    this.setState({sourceImg:null, croppedImg:null, imgIsEmpty:true, removing:false});
                    URL.revokeObjectURL(this.tempImage);
                    this.tempImageType = '';
                    this.sourceImageSize = [0,0];
                    this.sourceImageInitialScaleRatio = 1;
                    this.sourceImageCrop = null;
                })
                .catch((data) => { console.error(data); this.setState({removing:false}); })
        }
    };

    render() {
        return <div className={this.props.containerClass}>
            <div
                className={this.props.previewClass+'-dragndrop-wrapper'+(this.state.dragging ? ' dragging':'')}
                onDrop={this.handleFileDrop} onDragEnter={this.handleDragEnter} onDragOver={this.handleDragOver} onDragLeave={this.handleDragLeave}>
                {!this.state.imgIsEmpty && this.state.croppedImg &&
                <div className={this.props.previewClass+'-image-wrapper'} style={{'max-width':this.props.maxWidth}}>
                    <img
                        src={this.state.croppedImg}
                        onClick={this.handleRequestOpen}
                        className={this.props.previewClass}
                        style={{'max-width':this.props.maxWidth,'cursor':this.state.sourceImg ? 'pointer':'default'}}
                        title={this.state.sourceImg ? 'Изменить кадрирование':''}
                    />
                    <a
                        href="#"
                        className={this.props.linkClass+' remove-image'+(this.state.removing ? ' loading':'')}
                        onClick={this.handleRemove}
                    >
                        {gettext('image_uploader_remove')}
                    </a>
                </div>}
                {this.state.imgIsEmpty && <img
                    src={this.state.defaultImg}
                    className={this.props.previewClass+' '+this.props.previewClass+'-image-wrapper'}
                    style={{'max-width':this.props.maxWidth}}
                />}
                <div className={this.props.containerClass+'-input'}>
                    <FileInput
                        ref={this.setInputRef}
                        handleChange={this.handleFileChange}
                        acceptMime="image/*"
                        title={gettext('image_uploader_input')}
                        className={this.props.linkClass}
                    />{gettext('image_uploader_dragndrop')}
                </div>
            </div>
            <Modal
                header={this.tempImageCrop && !this.state.uploading ? gettext('image_uploader_change_crop') : this.props.title}
                onRequestClose={this.handleRequestClose}
                isOpen={this.state.editorOpened}
            >
                <Cropper
                    ref={this.setEditorRef}
                    style={{ 'height': '50vh', 'width': (this.tempImageSize[0]/this.tempImageSize[1]) * 50 + 'vh', 'max-width': '90vw'}}
                    src={this.tempImage}
                    viewMode={2}
                    responsive={true}
                    minCropBoxHeight={this.props.minImageWidth}
                    minCropBoxWidth={this.props.minImageHeight}
                    autoCropArea={1}
                    guides={true}
                    aspectRatio={this.props.aspectRatio}
                    dragMode={'move'}
                    data={this.tempImageCrop}
                    ready={this.onCropperReady}
                    zoom={this.onCropperZoom}
                />
                <br />
                <div className="modal-actions">
                    <button
                        className={'button button-primary'+(this.state.uploading ? ' loading':'')}
                        onClick={this.handleSave}
                    >
                        {gettext('image_uploader_save')}
                    </button>
                </div>
            </Modal>
        </div>
    }
}
